// -*- C++ -*-
#ifndef HERWIG_VectorMesonVectorScalarDecayer_H
#define HERWIG_VectorMesonVectorScalarDecayer_H
//
// This is the declaration of the VectorMesonVectorScalarDecayer class.
//
#include "Herwig++/Decay/DecayIntegrator.h"
#include "Herwig++/Decay/DecayPhaseSpaceMode.h"
#include "VectorMesonVectorScalarDecayer.fh"

namespace Herwig {
using namespace Herwig;

/** \ingroup Decay
 *
 *  This class is designed for the decay of a vector meson to a spin-1
 *  particle, either a vector meson or a photon, and a scalar meson.
 *  The current for the decay is
 *
 *  \f[\mathcal{M}=g\epsilon_mu\left[ p_V \cdot p_0 \epsilon_V^\mu  
 *                     -p_V^\mu \epsilon_V \cdot p_0\right],\f]
 *  where \f$\epsilon\f$ is the polarization vector of the decaying vector
 *  meson, \f$\epsilon_V\f$ is the polarization vector of the outgoing vector meson,
 *  \f$p_0\f$ is the momentum of the decaying particle and \f$p_V\f$ is the momentum
 *  of the outgoing vector meson.
 *
 * @see DecayIntegrator 
 * @see \ref VectorMesonVectorScalarDecayerInterfaces "The interfaces"
 * defined for VectorMesonVectorScalarDecayer.
 * 
 *  \author Peter Richardson
 * 
 */
class VectorMesonVectorScalarDecayer: public DecayIntegrator {

public:

  /** @name Standard constructors and destructors. */
  //@{
  /**
   * Default constructor.
   */
  VectorMesonVectorScalarDecayer();

  /**
   * Copy-constructor.
   */
  inline VectorMesonVectorScalarDecayer(const VectorMesonVectorScalarDecayer &);

  /**
   * Destructor.
   */
  virtual ~VectorMesonVectorScalarDecayer();
  //@}

public:

  /**
   * Which of the possible decays is required
   * @param cc Is this mode the charge conjugate
   * @param dm The decay mode
   */
  virtual int modeNumber(bool & cc,const DecayMode & dm) const;

  /**
   * Return the matrix element squared for a given mode and phase-space channel.
   * @param vertex Output the information on the vertex for spin correlations
   * @param ichan The channel we are calculating the matrix element for. 
   * @param part The decaying Particle.
   * @param decay The particles produced in the decay.
   * @return The matrix element squared for the phase-space configuration.
   */
  double me2(bool vertex, const int ichan,const Particle & part,
	     const ParticleVector & decay) const;

  /**
   * Output the setup information for the particle database
   * @param os The stream to output the information to
   * @param header Whether or not to output the information for MySQL
   */
  virtual void dataBaseOutput(ofstream & os,bool header) const;

public:

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * Standard Init function used to initialize the interfaces.
   */
  static void Init();

  /**
   * Specify the \f$1\to2\f$ matrix element to be used in the running width calculation.
   * @param dm The DecayMode
   * @param mecode The code for the matrix element as described
   *               in the GenericWidthGenerator class, in this case 4.
   * @param coupling The coupling for the matrix element.
   * @return True or False if this mode can be handled.
   */
  bool twoBodyMEcode(const DecayMode & dm, int & mecode, double & coupling) const;

protected:

  /** @name Clone Methods. */
  //@{
  /**
   * Make a simple clone of this object.
   * @return a pointer to the new object.
   */
  virtual IBPtr clone() const;

  /** Make a clone of this object, possibly modifying the cloned object
   * to make it sane.
   * @return a pointer to the new object.
   */
  virtual IBPtr fullclone() const;
  //@}

protected:

  /** @name Standard Interfaced functions. */
  //@{
  /**
   * Initialize this object after the setup phase before saving and
   * EventGenerator to disk.
   * @throws InitException if object could not be initialized properly.
   */
  virtual void doinit() throw(InitException);

  /**
   * Initialize this object to the begining of the run phase.
   */
  inline virtual void doinitrun();
  //@}

private:

  /**
   * Describe a concrete class with persistent data.
   */
  static ClassDescription<VectorMesonVectorScalarDecayer> initVectorMesonVectorScalarDecayer;

  /**
   * Private and non-existent assignment operator.
   */
  VectorMesonVectorScalarDecayer & operator=(const VectorMesonVectorScalarDecayer &);

private:

  /**
   * coupling for a decay
   */
  vector<InvEnergy> _coupling;

  /**
   * PDG codes for the incoming particles
   */
  vector<int> _incoming;

  /**
   * PDG codes for the outgoing vector
   */
  vector<int> _outgoingV;

  /**
   * PDG codes for the outgoing scalar mesons.
   */
  vector<int> _outgoingS;

  /**
   * maximum weight for a decay
   */
  vector<double> _maxweight;

  /**
   *  Initial size of the vectors
   */
  unsigned int _initsize;

};

}


#include "ThePEG/Utilities/ClassTraits.h"

namespace ThePEG {

/**
 * The following template specialization informs ThePEG about the
 * base class of VectorMesonVectorScalarDecayer.
 */
template <>
 struct BaseClassTrait<Herwig::VectorMesonVectorScalarDecayer,1> {
    /** Typedef of the base class of VectorMesonVectorScalarDecayer. */
  typedef Herwig::DecayIntegrator NthBase;
};

/**
 * The following template specialization informs ThePEG about the
 * name of this class and the shared object where it is defined.
 */
template <>
struct ClassTraits<Herwig::VectorMesonVectorScalarDecayer>
  : public ClassTraitsBase<Herwig::VectorMesonVectorScalarDecayer> {
  /** Return the class name.*/
  static string className() { return "Herwig++::VectorMesonVectorScalarDecayer"; }
  /**
   * Return the name of the shared library to be loaded to get
   * access to this class and every other class it uses
   * (except the base class).
   */
  static string library() { return "HwVMDecay.so"; }

};

}

#include "VectorMesonVectorScalarDecayer.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "VectorMesonVectorScalarDecayer.tcc"
#endif

#endif /* HERWIG_VectorMesonVectorScalarDecayer_H */
